import * as i0 from '@angular/core';
import { Injectable } from '@angular/core';
import * as i1 from 'mtm-auto';

let Base$2 = class Base {
    _mtmAuto = null;
    setAuto(service) {
        this._mtmAuto = service;
        return this;
    }
    getAuto() {
        return this._mtmAuto;
    }
};

let Base$1 = class Base {
    _mtmAuto = null;
    _toolObj = null;
    _strKey = "";
    setAuto(service) {
        this._mtmAuto = service;
        return this;
    }
    getAuto() {
        return this._mtmAuto;
    }
    setDeps(toolObj) {
        this._toolObj = toolObj;
        return this;
    }
    set(str) {
        //get rid of the different variations of line breaks so data
        //does not depend on the platform and can be compared as strings
        this._strKey = str.trim();
        this.initialize();
        return this;
    }
    get() {
        return this._strKey;
    }
    getTool() {
        return this._toolObj;
    }
    getTypeName() {
        return this._typeName;
    }
};

let PrivateKey$1 = class PrivateKey extends Base$1 {
    _typeName = "rsa-private";
    _passPhrase = null;
    _arrBuff = null;
    _wcaObjs = {}; //web crypto api objs
    async initialize() {
        const head = "-----BEGIN PRIVATE KEY-----";
        const foot = "-----END PRIVATE KEY-----";
        const pem = this.get().substring(head.length, this.get().length - foot.length);
        this._arrBuff = this.getAuto().get("utilities").getStrings().getArrays().toBuffer(window.atob(pem));
        return this;
    }
    setPassPhrase(str) {
        this._passPhrase = str;
        return this;
    }
    getPassPhrase() {
        return this._passPhrase;
    }
    async decrypt(data, pad = "RSA-OAEP", hash = "SHA-256") {
        return this.getTool().decrypt(this, data, pad, hash);
    }
    async getPublicKey() {
        return await this.getTool().getPublicKeyFromPrivateKey(this);
    }
    async sign(strData, pad = "RSASSA-PKCS1-v1_5", hash = "SHA-256") {
        return this.getTool().sign(this, strData, pad, hash);
    }
    async getBits() {
        var wcaObj = await this.getWcaObj("RSASSA-PKCS1-v1_5", "SHA-256", ["sign"]);
        return wcaObj.algorithm.modulusLength;
    }
    getArrayBuffer() {
        return this._arrBuff;
    }
    async getWcaObj(algoName, hashName, usage = []) {
        var hash = await this.getAuto().get("utilities").getHashing().getSHA().get256(algoName.toLowerCase() + hashName.toLowerCase() + JSON.stringify(usage));
        if (typeof this._wcaObjs[hash] == "undefined") {
            this._wcaObjs[hash] = await this.getTool().getWcaObj(this, algoName, hashName, usage);
        }
        return this._wcaObjs[hash];
    }
};

let PublicKey$1 = class PublicKey extends Base$1 {
    _typeName = "rsa-public";
    _arrBuff = null;
    _wcaObjs = {}; //web crypto api objs
    async initialize() {
        const head = "-----BEGIN PUBLIC KEY-----";
        const foot = "-----END PUBLIC KEY-----";
        const pem = this.get().substring(head.length, this.get().length - foot.length);
        this._arrBuff = this.getAuto().get("utilities").getStrings().getArrays().toBuffer(window.atob(pem));
        return this;
    }
    async encrypt(data, pad = "RSA-OAEP", hash = "SHA-256") {
        //to decrypt in PHP using default ssl bindings, hash must be "SHA-1"
        return this.getTool().encrypt(this, data, pad, hash);
    }
    async validateSignature(strData, signature, pad = "RSASSA-PKCS1-v1_5", hash = "SHA-256") {
        return this.getTool().verifySign(this, strData, signature, pad, hash);
    }
    getArrayBuffer() {
        return this._arrBuff;
    }
    async getWcaObj(algoName, hashName, usage = []) {
        var hash = await this.getAuto().get("utilities").getHashing().getSHA().get256(algoName.toLowerCase() + hashName.toLowerCase() + JSON.stringify(usage));
        if (typeof this._wcaObjs[hash] == "undefined") {
            this._wcaObjs[hash] = await this.getTool().getWcaObj(this, algoName, hashName, usage);
        }
        return this._wcaObjs[hash];
    }
    async getBits() {
        var wcaObj = await this.getWcaObj("RSASSA-PKCS1-v1_5", "SHA-256", ["verify"]);
        return wcaObj.algorithm.modulusLength;
    }
};

class BaseTool {
    _mtmAuto = null;
    setAuto(service) {
        this._mtmAuto = service;
        return this;
    }
    getAuto() {
        return this._mtmAuto;
    }
}

let RSA$1 = class RSA extends BaseTool {
    async encrypt(keyObj, strData, pad = "RSA-OAEP", hash = "SHA-256") {
        if (typeof crypto.subtle === "undefined") {
            throw this.getAuto().newException("Browser crypto is only available in a secure context (https)");
        }
        else if (keyObj.getTypeName() !== "rsa-public") {
            throw this.getAuto().newException("Cannot encrypt using key type: " + keyObj.getTypeName());
        }
        const key = await keyObj.getWcaObj(pad, hash, ["encrypt"]);
        const dataBuff = this.getAuto().get("utilities").getStrings().getArrays().toBuffer(strData);
        const result = await crypto.subtle.encrypt({ name: pad }, key, dataBuff);
        return this.getAuto().get("utilities").getArrays().getStrings().fromBuffer(result);
    }
    async decrypt(keyObj, encData, pad = "RSA-OAEP", hash = "SHA-256") {
        if (typeof crypto.subtle === "undefined") {
            throw this.getAuto().newException("Browser crypto is only available in a secure context (https)");
        }
        if (keyObj.getTypeName() !== "rsa-private") {
            throw this.getAuto().newException("Cannot decrypt using key type: " + keyObj.getTypeName());
        }
        const key = await keyObj.getWcaObj(pad, hash, ["decrypt"]);
        const dataBuff = this.getAuto().get("utilities").getStrings().getArrays().toBuffer(encData);
        const result = await crypto.subtle.decrypt({ name: pad }, key, dataBuff);
        return this.getAuto().get("utilities").getArrays().getStrings().fromBuffer(result);
    }
    async sign(keyObj, strData, pad = "RSASSA-PKCS1-v1_5", hash = "SHA-256") {
        if (typeof crypto.subtle === "undefined") {
            throw this.getAuto().newException("Browser crypto is only available in a secure context (https)");
        }
        else if (keyObj.getTypeName() !== "rsa-private") {
            throw this.getAuto().newException("Cannot sign using key type: " + keyObj.getTypeName());
        }
        const key = await keyObj.getWcaObj(pad, hash, ["sign"]);
        const dataBuff = this.getAuto().get("utilities").getStrings().getArrays().toBuffer(strData);
        const result = await crypto.subtle.sign(pad, key, dataBuff);
        return this.getAuto().get("utilities").getArrays().getStrings().fromBuffer(result);
    }
    async verifySign(keyObj, strData, sig, pad = "RSASSA-PKCS1-v1_5", hash = "SHA-256") {
        if (typeof crypto.subtle === "undefined") {
            throw this.getAuto().newException("Browser crypto is only available in a secure context (https)");
        }
        else if (keyObj.getTypeName() !== "rsa-public") {
            throw this.getAuto().newException("Cannot sign using key type: " + keyObj.getTypeName());
        }
        const key = await keyObj.getWcaObj(pad, hash, ["verify"]);
        const dataBuff = this.getAuto().get("utilities").getStrings().getArrays().toBuffer(strData);
        const sigBuff = this.getAuto().get("utilities").getStrings().getArrays().toBuffer(sig);
        const result = await crypto.subtle.verify(pad, key, sigBuff, dataBuff);
        return result;
    }
    async createPrivateKey(bits = 2048, passPhrase = "") {
        //usage, hash whatever is irelevant as we are about to convert to a PEM
        let keyPair = await window.crypto.subtle.generateKey({
            name: "RSASSA-PKCS1-v1_5",
            modulusLength: bits,
            publicExponent: new Uint8Array([0x01, 0x00, 0x01]),
            hash: { name: "SHA-256" }
        }, true, ["sign", "verify"]);
        const priKey = await crypto.subtle.exportKey("pkcs8", keyPair.privateKey);
        var priBin = this.getAuto().get("utilities").getArrays().getStrings().fromBuffer(priKey);
        var priPem = "-----BEGIN PRIVATE KEY-----\n";
        priPem += window.btoa(priBin).match(/.{1,64}/g).join("\n");
        priPem += "\n-----END PRIVATE KEY-----";
        var keyObj = this.getAuto().get("encrypt").getRSA().getPrivateKey(priPem);
        return keyObj;
    }
    async getPublicKeyFromPrivateKey(keyObj) {
        if (typeof crypto.subtle === "undefined") {
            throw this.getAuto().newException("Browser crypto is only available in a secure context (https)");
        }
        else if (keyObj.getTypeName() !== "rsa-private") {
            throw this.getAuto().newException("Not handled for type: " + keyObj.getTypeName());
        }
        //usage, hash whatever is irelevant as we are about to convert to a PEM
        const key = await keyObj.getWcaObj("RSA-OAEP", "SHA-256", ["decrypt"]);
        const jwk = await crypto.subtle.exportKey("jwk", key);
        delete jwk.d;
        delete jwk.dp;
        delete jwk.dq;
        delete jwk.q;
        delete jwk.qi;
        jwk.key_ops = ["encrypt"];
        const wcaObj = await crypto.subtle.importKey("jwk", jwk, {
            name: "RSA-OAEP",
            hash: { name: "SHA-256" }
        }, true, ["encrypt"]);
        const pubKey = await crypto.subtle.exportKey("spki", wcaObj);
        var pubBin = this.getAuto().get("utilities").getArrays().getStrings().fromBuffer(pubKey);
        var pubPem = "-----BEGIN PUBLIC KEY-----\n";
        pubPem += window.btoa(pubBin).match(/.{1,64}/g).join("\n");
        pubPem += "\n-----END PUBLIC KEY-----";
        var keyObj = this.getAuto().get("encrypt").getRSA().getPublicKey(pubPem);
        return keyObj;
    }
    async getWcaObj(keyObj, algoName, hashName, usage = []) {
        //never use the same key for more than one thing, but its up to you
        //"sign", "verify", "encrypt", "decrypt", "deriveKey", "deriveBits", "wrapKey", "unwrapKey"
        try {
            if (typeof crypto.subtle === "undefined") {
                throw this.getAuto().newException("Browser crypto is only available in a secure context (https)");
            }
            else if (keyObj.getTypeName() == "rsa-private") {
                var wcaObj = await window.crypto.subtle.importKey("pkcs8", keyObj.getArrayBuffer(), {
                    name: algoName,
                    hash: { name: hashName }
                }, true, usage);
                return wcaObj;
            }
            else if (keyObj.getTypeName() == "rsa-public") {
                var wcaObj = await window.crypto.subtle.importKey("spki", keyObj.getArrayBuffer(), {
                    name: algoName,
                    hash: { name: hashName }
                }, true, usage);
                return wcaObj;
            }
            else {
                throw this.getAuto().newException("Not handled for key type: " + keyObj.getTypeName());
            }
        }
        catch (errObj) {
            throw errObj;
        }
    }
};

class RSA extends Base$2 {
    _toolObj = null;
    getPrivateKey(strKey = null, passPhrase = null) {
        var rObj = new PrivateKey$1();
        rObj.setAuto(this.getAuto());
        rObj.setDeps(this.getTool());
        if (strKey !== null) {
            rObj.set(strKey);
        }
        if (passPhrase !== null) {
            rObj.setPassPhrase(passPhrase);
        }
        return rObj;
    }
    getPublicKey(strKey = null) {
        var rObj = new PublicKey$1();
        rObj.setAuto(this.getAuto());
        rObj.setDeps(this.getTool());
        if (strKey !== null) {
            rObj.set(strKey);
        }
        return rObj;
    }
    getTool() {
        if (this._toolObj === null) {
            this._toolObj = new RSA$1();
            this._toolObj.setAuto(this.getAuto());
        }
        return this._toolObj;
    }
}

class Base {
    _mtmAuto = null;
    _toolObj = null;
    _strKey = null;
    setAuto(service) {
        this._mtmAuto = service;
        return this;
    }
    getAuto() {
        return this._mtmAuto;
    }
    setDeps(toolObj) {
        this._toolObj = toolObj;
        return this;
    }
    set(str) {
        //get rid of the different variations of line breaks so data
        //does not depend on the platform and can be compared as strings
        this._strKey = str.trim();
        return this;
    }
    get() {
        return this._strKey;
    }
    getTool() {
        return this._toolObj;
    }
}

class PrivateKey extends Base {
    _passPhrase = null;
    setPassPhrase(str) {
        this._passPhrase = str;
        return this;
    }
    getPassPhrase() {
        return this._passPhrase;
    }
    sign(str, algo = "OPENSSL_ALGO_SHA1") {
        return this.getTool().sign(this, str, algo);
    }
}

class PublicKey extends Base {
    encrypt(data, pad = "OPENSSL_PKCS1_PADDING") {
        return this.getTool().encrypt(this, data, pad);
    }
    validateSignature(strData, signature, algo = "OPENSSL_ALGO_SHA1") {
        return this.getTool().verifySign(this, strData, signature, algo);
    }
}

let EC$1 = class EC extends BaseTool {
    _cryptoObj = null;
    setDeps(cryptoObj) {
        this._cryptoObj = cryptoObj;
        return this;
    }
    //	public encrypt(keyObj, strData, pad="OPENSSL_PKCS1_PADDING")
    //	{
    //		return this._cryptoObj.AES.encrypt(strData, keyObj.get());
    //	}
    sign(keyObj, str, algo = "OPENSSL_ALGO_SHA1") {
        //		console.log(keyObj);
        //		console.log(str);
        //		console.log(algo);
        //		var ec 		= new this._cryptoObj.ec();
        //		console.log(ec);
        //		var what	= ec.keyPair(null, keyObj.get());
        //		console.log(what);
    }
    verifySign(keyObj, strData, sig, algo = "OPENSSL_ALGO_SHA1") {
    }
};

class EC extends Base$2 {
    _cryptoObj = null;
    _toolObj = null;
    setDeps(cryptoObj) {
        this._cryptoObj = cryptoObj;
        return this;
    }
    getPrivateKey(strKey = null, passPhrase = null) {
        var rObj = new PrivateKey();
        rObj.setAuto(this.getAuto());
        rObj.setDeps(this.getTool());
        if (strKey !== null) {
            rObj.set(strKey);
        }
        if (passPhrase !== null) {
            rObj.setPassPhrase(passPhrase);
        }
        return rObj;
    }
    getPublicKey(strKey = null) {
        var rObj = new PublicKey();
        rObj.setAuto(this.getAuto());
        rObj.setDeps(this.getTool());
        if (strKey !== null) {
            rObj.set(strKey);
        }
        return rObj;
    }
    getTool() {
        if (this._toolObj === null) {
            this._toolObj = new EC$1();
            this._toolObj.setAuto(this.getAuto());
            this._toolObj.setDeps(this._cryptoObj);
        }
        return this._toolObj;
    }
}

class MtmEncrypt {
    _mtmAuto;
    _rsaFact = null;
    _ecFact = null;
    constructor(_mtmAuto) {
        this._mtmAuto = _mtmAuto;
        this._mtmAuto.set("encrypt", this);
    }
    getAuto() {
        return this._mtmAuto;
    }
    getRSA() {
        if (this._rsaFact === null) {
            this._rsaFact = new RSA();
            this._rsaFact.setAuto(this.getAuto());
        }
        return this._rsaFact;
    }
    getEC() {
        if (this._ecFact === null) {
            this._ecFact = new EC();
            this._ecFact.setAuto(this.getAuto());
        }
        return this._ecFact;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.1", ngImport: i0, type: MtmEncrypt, deps: [{ token: i1.MtmAuto }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.1.1", ngImport: i0, type: MtmEncrypt, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.1", ngImport: i0, type: MtmEncrypt, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.MtmAuto }] });

/*
 * Public API Surface of mtm-encrypt
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MtmEncrypt };
//# sourceMappingURL=mtm-encrypt.mjs.map
